#!/usr/bin/python2.7
# -*- coding: utf-8 -*-
#
# Univention Zimbra Connector
#  Univention Directory Listener script for the zimbra4ucs
#
# Copyright 2017 Univention GmbH
#
# http://www.univention.de/
#
# All rights reserved.
#
# The source code of this program is made available
# under the terms of the GNU Affero General Public License version 3
# (GNU AGPL V3) as published by the Free Software Foundation.
#
# Binary versions of this program provided by Univention to you as
# well as other copyrighted, protected or trademarked materials like
# Logos, graphics, fonts, specific documentations and configurations,
# cryptographic keys etc. are subject to a license agreement between
# you and Univention and not subject to the GNU AGPL V3.
#
# In the case you use this program under the terms of the GNU AGPL V3,
# the program is provided in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License with the Debian GNU/Linux or Univention distribution in file
# /usr/share/common-licenses/AGPL-3; if not, see
# <http://www.gnu.org/licenses/>.

from __future__ import absolute_import
import listener
import os
import time
import cPickle
import univention.debug

name = 'zimbra4ucs-user'
description = 'zimbra4ucs Connector'
filter = '(&(objectClass=univentionMail)(objectClass=zimbra4ucsUser))'
attributes = []
modrdn = '1'
directory = listener.configRegistry.get('zimbra4ucs/user/listener/dir', '/var/lib/zimbra4ucs-user-listener')


def _load_old_object(directory):
	filename = os.path.join(directory, 'tmp', 'old_dn')
	with open(filename) as f:
		p = cPickle.Unpickler(f)
		data = p.load()
	return data


def _dump_object_to_file(data, tmp=False):
	if tmp:
		filename = os.path.join(directory, 'tmp', 'old_dn')
	else:
		filename = os.path.join(directory, '%f' % time.time())
	with open(filename, 'w+') as f:
		os.chmod(filename, 0600)
		p = cPickle.Pickler(f)
		p.dump(data)
		p.clear_memo()


def handler(dn, new, old, command):
	univention.debug.debug(univention.debug.LISTENER, univention.debug.PROCESS, '%s: process %s for %s' % (name, command, dn))

	if listener.configRegistry.is_true('zimbra4ucs/user/listener/disabled', False):
		univention.debug.debug(univention.debug.LISTENER, univention.debug.INFO, '%s: connector disabled by UCR' % name)
		return

	listener.setuid(0)
	try:
		if not os.path.exists(os.path.join(directory, 'tmp')):
			os.makedirs(os.path.join(directory, 'tmp'))

		old_dn = None
		old_object = {}
		if os.path.exists(os.path.join(directory, 'tmp', 'old_dn')):
			(old_dn, old_object) = _load_old_object(directory)
			univention.debug.debug(univention.debug.LISTENER, univention.debug.PROCESS, '%s: got old object for %s' % (name, old_dn))

		if command == 'r':
			univention.debug.debug(univention.debug.LISTENER, univention.debug.PROCESS, '%s: modrdn for %s' % (name, dn))
			_dump_object_to_file((dn, old), tmp=True)
		else:
			# Normally we see two steps for the modrdn operation. But in case of the selective replication we
			# might only see the first step.
			#  https://forge.univention.org/bugzilla/show_bug.cgi?id=32542
			if old_dn and old_object and not old:
				old = old_object
				if new.get('entryUUID') != old_object.get('entryUUID'):
					univention.debug.debug(
						univention.debug.LISTENER, univention.debug.PROCESS,
						"The entryUUID attribute of the saved object (%s) does not match the \
						entryUUID attribute of the current object (%s). This can be normal in \
						a selective replication scenario." % (old_dn, dn)
					)
					# this is not the same object, simluate a delete
					dn = old_dn
					new = {}
					old_dn = None
			univention.debug.debug(univention.debug.LISTENER, univention.debug.PROCESS, '%s: save change for %s' % (name, dn))
			_dump_object_to_file((dn, new, old, old_dn))

			if os.path.exists(os.path.join(directory, 'tmp', 'old_dn')):
				os.unlink(os.path.join(directory, 'tmp', 'old_dn'))
	finally:
		listener.unsetuid()


def clean():
	listener.setuid(0)
	try:
		if os.path.exists(directory):
			for filename in os.listdir(directory):
				if filename != "tmp":
					os.remove(os.path.join(directory, filename))
			if os.path.exists(os.path.join(directory, 'tmp')):
				for filename in os.listdir(os.path.join(directory, 'tmp')):
					os.remove(os.path.join(directory, 'tmp', filename))
	finally:
		listener.unsetuid()


def postrun():
	pass


def initialize():
	clean()
